﻿##############################################################################
##
## Power Shell Module for managing Proxy Gateway Server
##
## Utilities to work with Gateway Administration Object
##
##############################################################################

# Imports
Import-Module Proxy-Utils
Import-Module Proxy-Constants
Import-Module Proxy-Gateway-ADs

# Test if Gateway Administration Control Object has the right properties and events
# Otherwise its an instance of a COM object and not of .NET interop assembly
function Test-GatewayObject($gateway)
{
    if( 
        -not $gateway `
        -or `
        -not ($gateway.GetType().FullName -eq 'ProxySDKv10.ProxyGWClass') `
        -or `
        -not ($gateway.currentStation))
    {
        Write-Error -message "Error: Gateway SDK Object is not an instance of a COM object and not of .NET interop assembly." 
        return $false
    }

    return $true
}

# Returns an object representing top-level group with given ID (IDs are defined in Proxy-Constants module as "groupID...")
function Get-TopLevelGroup ($gateway, $groupID)
{
    return $gateway.getGroups() | Where-Object {$_.groupID -eq $groupID}
}

# Returns an object representing a first group with given name
function Find-GroupByName($gateway, [string]$name)
{
    return $gateway.getAllGroups() | Where-Object {$_.groupName -eq $name} | Select -First 1
}

# Returns an object representing group with given ID (as a GUID string in curly braces)
function Find-GroupByID($gateway, $groupID)
{
    return $gateway.getAllGroups() | Where-Object {$_.groupID -eq $groupID}
}

# Copy Host to Groups with the same names as Organizational Units to wich Host belongs according to default AD
function Copy-HostToDesignatedOUs($gateway, $hostObject)
{
    if(-not (Test-GatewayObject $gateway))
    {
        return
    }
    
    # Get Host's OUs
    $adObj = Get-ComputerInDefaultAD $hostObject.machine
    if(-not $adObj)
    {
        Write-Warning -Message "Host machine: $hostObject.machine, wasn't found in default AD."
        return
    }

    # Test if Host is in Group with the same name as OU
    $OUs = ([adsi]($adObj.parent)).ou
    $adObj.Dispose()
    foreach($ou in $OUs)
    {
        # Find Gateway Group with the same name
        $group = $gateway.getAllGroups() | ? { $_.groupName -eq [string]$ou }
        if(-not $group)
        {
            Write-Warning -Message "Group: $ou, wasn't found on the Gateway."
            continue
        }

        # Check if Host is in the Group        
        if(-not ($group.getHosts() | ? { $_.machine -eq $hostObject.machine }))
        {
            # Add Host to Group
            $group.addHost($hostObject)
            Write-Host "Host: <$($hostObject.machine)> added to <$($group.groupName)> group."
        }
    }
}

# Copy Host to Groups from the provided list
function Copy-HostToGroups($gateway, $host_object, $group_names_list, $forceGroupCreation)
{
    if(-not ([bool]$group_names_list))
    {
        # Empty Group list - do nothing
        return
    }
    
    # First make sure the Host is managed
    if([int]$host_object.isManaged -eq 0)
    {
        $host_object.isManaged = -1
    }
    
    # Go through Groups in the list
    foreach($group_name in $group_names_list)
    {
        # Find Group with this name
        $group = Find-GroupByName -gateway $gateway -name $group_name
        if((-not $group) -and $forceGroupCreation)
        {
            # If Group with this name is not found and $forceGroupCreation == $true - create group
            #
            # NOTICE(v10): We are adding group to "Managed Hosts" group, which
            # maybe not what User wants. In the future this functionality should
            # either accept a groupID or to be moved to examples.
            $mnghostgroup = Get-TopLevelGroup -gateway $gateway -groupID $groupIDManagedHosts
            $group = $mnghostgroup.addGroup($group_name, "$group_name Group", $groupTypeNormal)
        }
        
        if($group)
        {
            # Check if Host is already in the Group
            if($group.getHosts() | where { $_.key -eq $host_object.key })
            {
                continue
            }
            
            # Copy Host to target group
            $group.addHost($host_object)
            Write-Host "Host: <$($host_object.machine)> added to <$group_name> group."
        }
    }
}

# Remove Host from Groups from the provided list
function Remove-HostFromGroups($gateway, $host_object, $group_names_list)
{
    if(-not ([bool]$group_names_list))
    {
        # Empty Group list - do nothing
        return
    }
    
    # First make sure the Host is managed
    if([int]$host_object.isManaged -eq 0)
    {
        # Unmanaged Host - do nothing
        return
    }
    
    # Go through Groups in the list
    foreach($group_name in $group_names_list)
    {
        # Find Group with this name
        $group = Find-GroupByName -gateway $gateway -name $group_name
        if($group)
        {
            # Check if Host is in the Group
            if($group.getHosts() | where { $_.key -eq $host_object.key })
            {
                $group.removeHost($host_object)
                Write-Host "Host: <$($host_object.machine)> removed from <$group_name> group."
            }
        }
    }
}

# Make sure Host is in the Groups from the provided list
function Sync-HostInGroups($gateway, $host_object, $group_names_list, $forceGroupCreation)
{
    $groups_intersection_list_and_gateway = @()
    
    #
    # Iterate through all Gateway's Groups
    #
    foreach($group in $gateway.getAllGroups())
    {
        # Look only at Normal Groups (not System, All Hosts, etc.)
        if($group.GroupType -ne $groupTypeNormal)
        {
            continue
        }
        
        $group_name = $group.groupName
        
        if($group_names_list -contains $group_name)
        {
            # If Group is in the list
            $groups_intersection_list_and_gateway += $group_name
            
            # Check if Host is not in the Group
            if(-not ($group.getHosts() | where { $_.key -eq $host_object.key }))
            {
                # Copy Host to the Group
                $group.addHost($host_object)
                Write-Host "Host: <$($host_object.machine)> added to <$group_name> group."
            }
        }
        else
        {
            # If Group is not in the list 
            
            # Check if Host is in the Group
            if($group.getHosts() | where { $_.key -eq $host_object.key })
            {
                # Remove Host from the Group
                $group.removeHost($host_object)
                Write-Host "Host: <$($host_object.machine)> removed from <$group_name> group."
            }            
        }
    }
    
    #
    # Iterate through all groups which are in the list but not in the Gateway
    #
    if(-not $forceGroupCreation)
    {
        return
    }
    
    foreach($group_name in $group_names_list)
    {
        if($groups_intersection_list_and_gateway -contains $group_name)
        {
            continue
        }
        
        # Create group
        #
        # NOTICE(v10): We are adding group to "Managed Hosts" group, which
        # maybe not what User wants. In the future this functionality should
        # either accept a groupID or to be moved to examples.
        $mnghostgroup = Get-TopLevelGroup -gateway $gateway -groupID $groupIDManagedHosts
        $group = $mnghostgroup.addGroup($group_name, "$group_name Group", $groupTypeNormal)
        if($group)
        {
            # Copy Host to target group
            $group.addHost($host_object)
            Write-Host "Host: <$($host_object.machine)> added to <$group_name> group."
        }
    }
}

# Iterates through all Managed and Unmanaged Hosts on the Gateway and calls $policy for each Host with Host Object as a parameter. 
# Next, it gets list of Group's Names returned from the $policy and makes sure this Host belongs to all those Groups, making it 
# Managed first if list is not empty.
function Copy-HostToGroupsByPolicy($gateway, $policy, $forceGroupCreation=$false)
{
    if(-not (Test-GatewayObject $gateway))
    {
        return
    }
    
    # NOTICE(v10): We are only doing this with the Hosts in My Hosts
    # group. In the future this functionality should either accept a 
    # groupID or to be moved to examples.
    $gateway.getHosts() `
    | where { $_.machine -ne 'virtual' } `
    | foreach { Copy-HostToGroups -gateway $gateway -host_object $_ -group_names_list (& $policy $_) -forceGroupCreation $forceGroupCreation }
}

# Removes Host from groups by given policy (works the same as Copy-HostToGroupsByPolicy but removing instead of copying).
function Remove-HostFromGroupsByPolicy($gateway, $policy)
{
    if(-not (Test-GatewayObject $gateway))
    {
        return
    }
    
    # NOTICE(v10): We are only doing this with the Hosts in My Hosts
    # group. In the future this functionality should either accept a 
    # groupID or to be moved to examples.
    $gateway.getHosts() `
    | where { ($_.machine -ne 'virtual') -and ($_.isManaged -eq -1) } `
    | foreach { Remove-HostFromGroups -gateway $gateway -host_object $_ -group_names_list (& $policy $_) }
}

# Iterates through all Managed and Unmanaged Hosts on the Gateway and calls $policy for each Host with Host Object as a parameter. 
# Next, it gets list of Group's Names returned from the $policy and makes sure this Host belongs to all those Groups which are 
# in the list, and doesn't belongs to the groups which are not in the list but exists on the Gateway.
function Sync-HostInGroupsByPolicy($gateway, $policy, $forceGroupCreation=$false)
{
    if(-not (Test-GatewayObject $gateway))
    {
        return
    }
    
    # NOTICE(v10): We are only doing this with the Hosts in My Hosts
    # group. In the future this functionality should either accept a 
    # groupID or to be moved to examples.
    $gateway.getHosts() `
    | where { $_.machine -ne 'virtual' } `
    | foreach { Sync-HostInGroups -gateway $gateway -host_object $_ -group_names_list (& $policy $_) -forceGroupCreation $forceGroupCreation }
}

#
# Module exports
#
Export-ModuleMember -Function Test-GatewayObject, Find-*, Copy-*, Remove-*, Sync-*, Get-*

# SIG # Begin signature block
# MIIXrQYJKoZIhvcNAQcCoIIXnjCCF5oCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUp/UiKc5Ikj7tUibaQstuVIR0
# vUigghKkMIID7jCCA1egAwIBAgIQfpPr+3zGTlnqS5p31Ab8OzANBgkqhkiG9w0B
# AQUFADCBizELMAkGA1UEBhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTEUMBIG
# A1UEBxMLRHVyYmFudmlsbGUxDzANBgNVBAoTBlRoYXd0ZTEdMBsGA1UECxMUVGhh
# d3RlIENlcnRpZmljYXRpb24xHzAdBgNVBAMTFlRoYXd0ZSBUaW1lc3RhbXBpbmcg
# Q0EwHhcNMTIxMjIxMDAwMDAwWhcNMjAxMjMwMjM1OTU5WjBeMQswCQYDVQQGEwJV
# UzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRpb24xMDAuBgNVBAMTJ1N5bWFu
# dGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EgLSBHMjCCASIwDQYJKoZIhvcN
# AQEBBQADggEPADCCAQoCggEBALGss0lUS5ccEgrYJXmRIlcqb9y4JsRDc2vCvy5Q
# WvsUwnaOQwElQ7Sh4kX06Ld7w3TMIte0lAAC903tv7S3RCRrzV9FO9FEzkMScxeC
# i2m0K8uZHqxyGyZNcR+xMd37UWECU6aq9UksBXhFpS+JzueZ5/6M4lc/PcaS3Er4
# ezPkeQr78HWIQZz/xQNRmarXbJ+TaYdlKYOFwmAUxMjJOxTawIHwHw103pIiq8r3
# +3R8J+b3Sht/p8OeLa6K6qbmqicWfWH3mHERvOJQoUvlXfrlDqcsn6plINPYlujI
# fKVOSET/GeJEB5IL12iEgF1qeGRFzWBGflTBE3zFefHJwXECAwEAAaOB+jCB9zAd
# BgNVHQ4EFgQUX5r1blzMzHSa1N197z/b7EyALt0wMgYIKwYBBQUHAQEEJjAkMCIG
# CCsGAQUFBzABhhZodHRwOi8vb2NzcC50aGF3dGUuY29tMBIGA1UdEwEB/wQIMAYB
# Af8CAQAwPwYDVR0fBDgwNjA0oDKgMIYuaHR0cDovL2NybC50aGF3dGUuY29tL1Ro
# YXd0ZVRpbWVzdGFtcGluZ0NBLmNybDATBgNVHSUEDDAKBggrBgEFBQcDCDAOBgNV
# HQ8BAf8EBAMCAQYwKAYDVR0RBCEwH6QdMBsxGTAXBgNVBAMTEFRpbWVTdGFtcC0y
# MDQ4LTEwDQYJKoZIhvcNAQEFBQADgYEAAwmbj3nvf1kwqu9otfrjCR27T4IGXTdf
# plKfFo3qHJIJRG71betYfDDo+WmNI3MLEm9Hqa45EfgqsZuwGsOO61mWAK3ODE2y
# 0DGmCFwqevzieh1XTKhlGOl5QGIllm7HxzdqgyEIjkHq3dlXPx13SYcqFgZepjhq
# IhKjURmDfrYwggSjMIIDi6ADAgECAhAOz/Q4yP6/NW4E2GqYGxpQMA0GCSqGSIb3
# DQEBBQUAMF4xCzAJBgNVBAYTAlVTMR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3Jh
# dGlvbjEwMC4GA1UEAxMnU3ltYW50ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBD
# QSAtIEcyMB4XDTEyMTAxODAwMDAwMFoXDTIwMTIyOTIzNTk1OVowYjELMAkGA1UE
# BhMCVVMxHTAbBgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMTQwMgYDVQQDEytT
# eW1hbnRlYyBUaW1lIFN0YW1waW5nIFNlcnZpY2VzIFNpZ25lciAtIEc0MIIBIjAN
# BgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAomMLOUS4uyOnREm7Dv+h8GEKU5Ow
# mNutLA9KxW7/hjxTVQ8VzgQ/K/2plpbZvmF5C1vJTIZ25eBDSyKV7sIrQ8Gf2Gi0
# jkBP7oU4uRHFI/JkWPAVMm9OV6GuiKQC1yoezUvh3WPVF4kyW7BemVqonShQDhfu
# ltthO0VRHc8SVguSR/yrrvZmPUescHLnkudfzRC5xINklBm9JYDh6NIipdC6Anqh
# d5NbZcPuF3S8QYYq3AhMjJKMkS2ed0QfaNaodHfbDlsyi1aLM73ZY8hJnTrFxeoz
# C9Lxoxv0i77Zs1eLO94Ep3oisiSuLsdwxb5OgyYI+wu9qU+ZCOEQKHKqzQIDAQAB
# o4IBVzCCAVMwDAYDVR0TAQH/BAIwADAWBgNVHSUBAf8EDDAKBggrBgEFBQcDCDAO
# BgNVHQ8BAf8EBAMCB4AwcwYIKwYBBQUHAQEEZzBlMCoGCCsGAQUFBzABhh5odHRw
# Oi8vdHMtb2NzcC53cy5zeW1hbnRlYy5jb20wNwYIKwYBBQUHMAKGK2h0dHA6Ly90
# cy1haWEud3Muc3ltYW50ZWMuY29tL3Rzcy1jYS1nMi5jZXIwPAYDVR0fBDUwMzAx
# oC+gLYYraHR0cDovL3RzLWNybC53cy5zeW1hbnRlYy5jb20vdHNzLWNhLWcyLmNy
# bDAoBgNVHREEITAfpB0wGzEZMBcGA1UEAxMQVGltZVN0YW1wLTIwNDgtMjAdBgNV
# HQ4EFgQURsZpow5KFB7VTNpSYxc/Xja8DeYwHwYDVR0jBBgwFoAUX5r1blzMzHSa
# 1N197z/b7EyALt0wDQYJKoZIhvcNAQEFBQADggEBAHg7tJEqAEzwj2IwN3ijhCcH
# bxiy3iXcoNSUA6qGTiWfmkADHN3O43nLIWgG2rYytG2/9CwmYzPkSWRtDebDZw73
# BaQ1bHyJFsbpst+y6d0gxnEPzZV03LZc3r03H0N45ni1zSgEIKOq8UvEiCmRDoDR
# EfzdXHZuT14ORUZBbg2w6jiasTraCXEQ/Bx5tIB7rGn0/Zy2DBYr8X9bCT2bW+IW
# yhOBbQAuOA2oKY8s4bL0WqkBrxWcLC9JG9siu8P+eJRRw4axgohd8D20UaF5Mysu
# e7ncIAkTcetqGVvP6KUwVyyJST+5z3/Jvz4iaGNTmr1pdKzFHTx/kuDDvBzYBHUw
# ggTQMIIDuKADAgECAgEHMA0GCSqGSIb3DQEBCwUAMIGDMQswCQYDVQQGEwJVUzEQ
# MA4GA1UECBMHQXJpem9uYTETMBEGA1UEBxMKU2NvdHRzZGFsZTEaMBgGA1UEChMR
# R29EYWRkeS5jb20sIEluYy4xMTAvBgNVBAMTKEdvIERhZGR5IFJvb3QgQ2VydGlm
# aWNhdGUgQXV0aG9yaXR5IC0gRzIwHhcNMTEwNTAzMDcwMDAwWhcNMzEwNTAzMDcw
# MDAwWjCBtDELMAkGA1UEBhMCVVMxEDAOBgNVBAgTB0FyaXpvbmExEzARBgNVBAcT
# ClNjb3R0c2RhbGUxGjAYBgNVBAoTEUdvRGFkZHkuY29tLCBJbmMuMS0wKwYDVQQL
# EyRodHRwOi8vY2VydHMuZ29kYWRkeS5jb20vcmVwb3NpdG9yeS8xMzAxBgNVBAMT
# KkdvIERhZGR5IFNlY3VyZSBDZXJ0aWZpY2F0ZSBBdXRob3JpdHkgLSBHMjCCASIw
# DQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBALngyxDUr3a91JNi6zBkuIEIbMME
# 2WIXji//PmXPj85i5jxSHNoWRUtVq3hrY4NikM4PaWyZyBoUi0zMRTPqiNyeo68r
# /oBhnXlXxM8u9D8wPF1H/JoWvMM3lkFRjhFLVPgovtCMvvAwOB7zsCb4Zkdjbd5x
# JkePOEdT0UYdtOPcAOpFrL28cdmqbwDb280wOnlPX0xH+B3vW8LEnWA7sbJDkdik
# M07qs9YnT60liqXG9NXQpq50BWRXiLVEVdQtKjo++Li96TIKApRkxBY6UPFKrud5
# M68MIAd/6N8EOcJpAmxjUvp3wRvIdIfIuZMYUFQ1S2lOvDvTSS4f3MHSUvsCAwEA
# AaOCARowggEWMA8GA1UdEwEB/wQFMAMBAf8wDgYDVR0PAQH/BAQDAgEGMB0GA1Ud
# DgQWBBRAwr0njsw0gzCiM9f7bLPwtCyAzjAfBgNVHSMEGDAWgBQ6moUHEGcotu/2
# vQVBbiDBlNoP3jA0BggrBgEFBQcBAQQoMCYwJAYIKwYBBQUHMAGGGGh0dHA6Ly9v
# Y3NwLmdvZGFkZHkuY29tLzA1BgNVHR8ELjAsMCqgKKAmhiRodHRwOi8vY3JsLmdv
# ZGFkZHkuY29tL2dkcm9vdC1nMi5jcmwwRgYDVR0gBD8wPTA7BgRVHSAAMDMwMQYI
# KwYBBQUHAgEWJWh0dHBzOi8vY2VydHMuZ29kYWRkeS5jb20vcmVwb3NpdG9yeS8w
# DQYJKoZIhvcNAQELBQADggEBAAh+bJMQyDi4lqmQS/+hX08E72w+nIgGyVCPpnP3
# VzEbvrzkL9v4utNb4LTn5nliDgyi12pjczG19ahIpDsILaJdkNe0fCVPEVYwxLZE
# nXssneVe5u8MYaq/5Cob7oSeuIN9wUPORKcTcA2RH/TIE62DYNnYcqhzJB61rCIO
# yheJYlhEG6uJJQEAD83EG2LbUbTTD1Eqm/S8c/x2zjakzdnYLOqum/UqspDRTXUY
# ij+KQZAjfVtL/qQDWJtGssNgYIP4fVBBzsKhkMO77wIv0hVU7kQV2Qqup4oz7bEt
# djYm3ATrn/dhHxXch2/uRpYoraEmfQoJpy4Eo428+LwEMAEwggUzMIIEG6ADAgEC
# AgkAlwn07EG7bckwDQYJKoZIhvcNAQELBQAwgbQxCzAJBgNVBAYTAlVTMRAwDgYD
# VQQIEwdBcml6b25hMRMwEQYDVQQHEwpTY290dHNkYWxlMRowGAYDVQQKExFHb0Rh
# ZGR5LmNvbSwgSW5jLjEtMCsGA1UECxMkaHR0cDovL2NlcnRzLmdvZGFkZHkuY29t
# L3JlcG9zaXRvcnkvMTMwMQYDVQQDEypHbyBEYWRkeSBTZWN1cmUgQ2VydGlmaWNh
# dGUgQXV0aG9yaXR5IC0gRzIwHhcNMTgwNTAyMTgwNjM3WhcNMjAwNjEyMTk0MTIz
# WjB0MQswCQYDVQQGEwJVUzEWMBQGA1UECBMNTWFzc2FjaHVzZXR0czEPMA0GA1UE
# BxMGQm9zdG9uMR0wGwYDVQQKExRQcm94eSBOZXR3b3JrcywgSW5jLjEdMBsGA1UE
# AxMUUHJveHkgTmV0d29ya3MsIEluYy4wggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAw
# ggEKAoIBAQDBajmeyJxUF2BLSCG5EI2XSjrkE3ac4LHOR4AJtSSnMFLiWQIph2Zf
# x5hIYB776GJFRXVieAPW+t3lmOf5RYIhbNc8xxIJ3/VSXZKBl7iTPDkc1EERwI8d
# Uw3mCw6QYkS2taY3pBhYSqTLloWeaWpETjj3euW5BZcSK3HKeHay6PpN92HhzdLL
# PTyAdH3TSr99QCLdyNzlTiSLaRMfEypZ87r7yK4QIU6sYKTSo7Tm1pUvkXeQIcQs
# tVnkAa3eeGFBjFQLQNpDhlZogwFipOJqWjiuDALO17+JoTgDggg63XRrVsgWdDow
# R2KGzwP6xXufP0McNJyGPOCxF1m9xP2BAgMBAAGjggGFMIIBgTAMBgNVHRMBAf8E
# AjAAMBMGA1UdJQQMMAoGCCsGAQUFBwMDMA4GA1UdDwEB/wQEAwIHgDA1BgNVHR8E
# LjAsMCqgKKAmhiRodHRwOi8vY3JsLmdvZGFkZHkuY29tL2dkaWcyczUtNC5jcmww
# XQYDVR0gBFYwVDBIBgtghkgBhv1tAQcXAjA5MDcGCCsGAQUFBwIBFitodHRwOi8v
# Y2VydGlmaWNhdGVzLmdvZGFkZHkuY29tL3JlcG9zaXRvcnkvMAgGBmeBDAEEATB2
# BggrBgEFBQcBAQRqMGgwJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmdvZGFkZHku
# Y29tLzBABggrBgEFBQcwAoY0aHR0cDovL2NlcnRpZmljYXRlcy5nb2RhZGR5LmNv
# bS9yZXBvc2l0b3J5L2dkaWcyLmNydDAfBgNVHSMEGDAWgBRAwr0njsw0gzCiM9f7
# bLPwtCyAzjAdBgNVHQ4EFgQUhXo2BaqNUEfl8vkqaW+RGSI/1JAwDQYJKoZIhvcN
# AQELBQADggEBAINdQ5UlRfjELvPD20WsfmQUTs6IxmAHBX87XaXv+OmhDTG2MJ8a
# cuCF3rsPFX2LNH+jQ8op4qlWdEjwyqdmgUh4b93gDI0RO1eo4b0/cuArcCW9tg7F
# lpwoi0zogmr/PcTzqql/flT64IQUFE9DpoENBnaXJyUJK2op5RixuxFeKRFcn6Vu
# hRlR5yAOydXtOf9vCcDrnLinlGtXGFhAKlOlXHf+vUsNH2tj4BJdOWzrtyf8dPDi
# EQNjil2EkBsIA6z3xvwN/rtqlirMaxZloYZjx61ZTqrnjZ7ZaHpDhw0ImhWoh9du
# HOh2qGF+YugB3jfE/VitbQWC4+GU5ncUqu8xggRzMIIEbwIBATCBwjCBtDELMAkG
# A1UEBhMCVVMxEDAOBgNVBAgTB0FyaXpvbmExEzARBgNVBAcTClNjb3R0c2RhbGUx
# GjAYBgNVBAoTEUdvRGFkZHkuY29tLCBJbmMuMS0wKwYDVQQLEyRodHRwOi8vY2Vy
# dHMuZ29kYWRkeS5jb20vcmVwb3NpdG9yeS8xMzAxBgNVBAMTKkdvIERhZGR5IFNl
# Y3VyZSBDZXJ0aWZpY2F0ZSBBdXRob3JpdHkgLSBHMgIJAJcJ9OxBu23JMAkGBSsO
# AwIaBQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEM
# BgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMCMGCSqG
# SIb3DQEJBDEWBBSUSluxNgBDEsCRNRa4OAy0oegmcjANBgkqhkiG9w0BAQEFAASC
# AQCLRdbGHLXG8jlIZ8+dKFWjQaPvbwVcafV7t4OO1Lrhxsu2aSiI24nkZWQMKJFN
# FBg9f/0Xx8Sto4anmRrjbm/QeStlUgetbzDiqtj3tm1Lp1syYiHLA+URuD3BZ6DP
# uKsineK2kDYkt/SVlh0FNz4f5TeHGndaojnfY4CP9Rm51gChmnwoBd1joeKQEblP
# /bB5+lesMNOGLlvYbqe4qRwwt1V78HeA0gtlf7/A97vreoI2FfPTLNEzcaRnm4Uw
# qlsmP4pFh4lDMtNJPsnRndjaM+M8iDs1PVBpqIhTjS1dyqJtMAPFQycTu4HA6Lb1
# HqJx0dX9tzUxKmAWIKdxJkgVoYICCzCCAgcGCSqGSIb3DQEJBjGCAfgwggH0AgEB
# MHIwXjELMAkGA1UEBhMCVVMxHTAbBgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9u
# MTAwLgYDVQQDEydTeW1hbnRlYyBUaW1lIFN0YW1waW5nIFNlcnZpY2VzIENBIC0g
# RzICEA7P9DjI/r81bgTYapgbGlAwCQYFKw4DAhoFAKBdMBgGCSqGSIb3DQEJAzEL
# BgkqhkiG9w0BBwEwHAYJKoZIhvcNAQkFMQ8XDTE4MTIxMjIwMDgxNlowIwYJKoZI
# hvcNAQkEMRYEFKaJRqH6H5CTSMHIVV/Ux8vn+yH+MA0GCSqGSIb3DQEBAQUABIIB
# AKG+28MSChW6eOqx8uJNd7COoqsYLPTi5S8o3h4iB6ZGL5YidN9lxJtDMPws5Th4
# gvpWi2wqIQUSAJH0G+1v3iUnJvjtCnZ3Ziy1+B8wdWT0tI8hferZGnPvawBlDhQ7
# jf3hMVMFyxHW6wrywgNQmiR5K3g1uRtdOMCzo6nSqfgcorHt3QRQaUWrUacck1hK
# LGCgeObngPc5WdRi1FPyYKh1n0YZgUoCbw/i6BLaVEHXWvHxv1TFfgFoOA2bEf09
# bBZL2Amc0UcJnZFTYSqw17c2SCd9u4W/+rs25nzibazlfCVMzj+h/Y4N1kDjioRv
# kbN6aLAa9XvUpMEmzzlOnAQ=
# SIG # End signature block
