﻿##############################################################################
##
## Power Shell Module for managing Proxy Gateway Security
##
## Exported functions:
##   Get-GatewayAuthorities
##   Find-UsersInAuthority
##   Find-GroupsInAuthority
##   Add-GatewayAccount
##   Remove-GatewayAccountByName
##   
## Exported examples:
##
##
##
##############################################################################


# Imports
Import-Module Proxy-Utils
Import-Module Proxy-Constants
Import-Module Proxy-Gateway-Utils


<#

.FUNCTION NAME

Get-GatewayAuthorities

.SYNOPSIS

Returns a list of "authorities" which can be used for importing User Accounts to
the Gateway Server. Authorities from this list can be used to call Find-UserAccounts.

NOTICE: When you run a PowerShell function which returns an enumeration, that function might return 0, 1 or N items.  
If you are assigning the results of that function to a variable, you will either get $null, 1 object or an array of objects respectively.  
Sometimes, you won’t care about the different types returned, but at the other times, you’ll actually prefer to force 
the results to always be an array. In those cases, you can use @(...) to wrap the return from the function.
Like this: $auth_list = @(Get-GatewayAuthorities -gateway $gws)


.RELATED
Find-UserAccounts

.EXAMPLE

PS > $auth_list = Get-GatewayAuthorities -gateway $gws

#>
function Get-GatewayAuthorities($gateway = $(throw "Please provide Gateway Object"))
{
    if(-not (Test-GatewayObject $gateway))
    {
        return
    }

    return $gateway.settings.gatewaySecurity.GetAuthorities()
}

<#

.FUNCTION NAME

Find-UsersInAuthority

.SYNOPSIS

Finds a list of User Accounts in given authority matching some prefix.
Use `Get-GatewayAuthorities` to retrieve a list of authorities.
Returns list of User Accounts and boolean indicating if there is more data available.
You can limit a number of returned results by setting `-limit` something else than 0.

.RELATED
Get-GatewayAuthorities

.EXAMPLE

PS > $user_list, $more_data = Find-UsersInAuthority -gateway $gws -authority $auth_list[0] -prefix "alex" -limit 0

#>
function Find-UsersInAuthority(
    $gateway = $(throw "Please provide Gateway Object"),
    [string]$authority = $(throw "Please provide authority name"),
    [string]$prefix = $(throw "Please provide some prefix for the user's name or an empty string"),
    [int]$limit = 0) # 0 - means no limit, any other number will just restrict number of returned results
{
    if(-not (Test-GatewayObject $gateway))
    {
        return
    }

    [ref] $moreData = 0
    $ret = $gateway.settings.gatewaySecurity.SearchAuthorityForUsers($authority, $prefix, $limit, $moreData) | ConvertTo-Array
    return $ret, [bool] $moreData.Value
}

<#

.FUNCTION NAME

Find-GroupsInAuthority

.SYNOPSIS

Finds a list of Groups in given authority matching some prefix.
Use `Get-GatewayAuthorities` to retrieve a list of authorities.
Returns list of User Accounts and boolean indicating if there is more data available.
You can limit a number of returned results by setting `-limit` something else than 0.

.RELATED
Get-GatewayAuthorities

.EXAMPLE

PS > $group_list, $more_data = Find-GroupsInAuthority -gateway $gws -authority $auth_list[0] -prefix "Usr" -limit 0

#>
function Find-GroupsInAuthority(
    $gateway = $(throw "Please provide Gateway Object"),
    [string]$authority = $(throw "Please provide authority name"),
    [string]$prefix = $(throw "Please provide some prefix for the group's name or an empty string"),
    [int]$limit = 0) # 0 - means no limit, any other number will just restrict number of returned results
{
    if(-not (Test-GatewayObject $gateway))
    {
        return
    }

    [ref] $moreData = 0
    $ret = $gateway.settings.gatewaySecurity.SearchAuthorityForGroups($authority, $prefix, $limit, $moreData) | ConvertTo-Array
    return $ret, [bool] $moreData.Value
}

<#

.FUNCTION NAME

Add-GatewayAccount

.SYNOPSIS

Imports User Account to the Gateway assigning it a "category" and linking it to the security of Groups and Hosts.

You can obtain a list of User Accounts by using Find-UsersInAuthority or Find-GroupsInAuthority.

Categories are: $usercatPersonal, $usercatMaster, $usercatAdministrator

To import an individual or group account as a Personal category user, you need workstation ID of the PROXY Host to be assigned to that account.  In that case the GroupIDs is left blank.  
To import an individual or group account as a Master category user, you need the ID of one or more PROXY Host groups. The Host group IDs are then constructed as a comma-separated list, e.g. "<group1.,<group2>,<etc.>", and in this case the WorkstationID is left blank.  
To import an individual or group account as a Administrative category account, leave both WorkstationID and GroupIDs blank.  
You can also import an account that has already been imported, and thereby change its category or reset its security permissions to a known state. 

.RELATED
Find-UsersInAuthority
Find-GroupsInAuthority

.EXAMPLE

PS > $user_list = Add-GatewayAccount -gateway $gws -account $user_list[0] -usercat $usercatPersonal -workstationID "w={14534C36-2E98-4F81-B3E0-38EFB15D8629}"

#>
function Add-GatewayAccount(
    $gateway = $(throw "Please provide Gateway Object"),
    $account = $(throw "Please provide User Account returned from Find-UsersInAuthority or Find-GroupsInAuthority"),
    $usercat = $(throw "Please provide Category for the User Account from the list in Proxy-Constants 'usercat...'"),
    [string[]]$groupIDs,
    [string]$workstationID)
{
    if(-not (Test-GatewayObject $gateway))
    {
        return
    }

    $groupIDsStr = ""
    if ($groupIDs)
    {
        $groupIDsStr = $groupIDs -join ","
    }

    if (-not $workstationID)
    {
        $workstationID = ""
    }

    return $gateway.settings.importAccount($account, $usercat, $groupIDsStr, $workstationID)
}

<#

.FUNCTION NAME

Remove-GatewayAccountByName

.SYNOPSIS

Removes User Account from the Gateway.

.RELATED

.EXAMPLE

PS >  Remove-GatewayAccountByName -gateway $gws -accountName "ralex@organization.com"

#>
function Remove-GatewayAccountByName (
    $gateway = $(throw "Please provide Gateway Object"),
    [string]$accountName = $(throw "Please provide Name for the User Account to remove"))
{
    if(-not (Test-GatewayObject $gateway))
    {
        return
    }

    $acc = $gw.settings.getAccounts() | Where-Object {$_.accountName -eq $accountName} | Select -First 1
    if ($acc)
    {
        $acc.deleteAccount()
    }
}

#
# Module exports
#
Export-ModuleMember -Function  Get-*, Find-*, Add-*, Remove-*

# SIG # Begin signature block
# MIIXrQYJKoZIhvcNAQcCoIIXnjCCF5oCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUQttJezJUe0t1qg760oOe28LN
# QxOgghKkMIID7jCCA1egAwIBAgIQfpPr+3zGTlnqS5p31Ab8OzANBgkqhkiG9w0B
# AQUFADCBizELMAkGA1UEBhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTEUMBIG
# A1UEBxMLRHVyYmFudmlsbGUxDzANBgNVBAoTBlRoYXd0ZTEdMBsGA1UECxMUVGhh
# d3RlIENlcnRpZmljYXRpb24xHzAdBgNVBAMTFlRoYXd0ZSBUaW1lc3RhbXBpbmcg
# Q0EwHhcNMTIxMjIxMDAwMDAwWhcNMjAxMjMwMjM1OTU5WjBeMQswCQYDVQQGEwJV
# UzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRpb24xMDAuBgNVBAMTJ1N5bWFu
# dGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EgLSBHMjCCASIwDQYJKoZIhvcN
# AQEBBQADggEPADCCAQoCggEBALGss0lUS5ccEgrYJXmRIlcqb9y4JsRDc2vCvy5Q
# WvsUwnaOQwElQ7Sh4kX06Ld7w3TMIte0lAAC903tv7S3RCRrzV9FO9FEzkMScxeC
# i2m0K8uZHqxyGyZNcR+xMd37UWECU6aq9UksBXhFpS+JzueZ5/6M4lc/PcaS3Er4
# ezPkeQr78HWIQZz/xQNRmarXbJ+TaYdlKYOFwmAUxMjJOxTawIHwHw103pIiq8r3
# +3R8J+b3Sht/p8OeLa6K6qbmqicWfWH3mHERvOJQoUvlXfrlDqcsn6plINPYlujI
# fKVOSET/GeJEB5IL12iEgF1qeGRFzWBGflTBE3zFefHJwXECAwEAAaOB+jCB9zAd
# BgNVHQ4EFgQUX5r1blzMzHSa1N197z/b7EyALt0wMgYIKwYBBQUHAQEEJjAkMCIG
# CCsGAQUFBzABhhZodHRwOi8vb2NzcC50aGF3dGUuY29tMBIGA1UdEwEB/wQIMAYB
# Af8CAQAwPwYDVR0fBDgwNjA0oDKgMIYuaHR0cDovL2NybC50aGF3dGUuY29tL1Ro
# YXd0ZVRpbWVzdGFtcGluZ0NBLmNybDATBgNVHSUEDDAKBggrBgEFBQcDCDAOBgNV
# HQ8BAf8EBAMCAQYwKAYDVR0RBCEwH6QdMBsxGTAXBgNVBAMTEFRpbWVTdGFtcC0y
# MDQ4LTEwDQYJKoZIhvcNAQEFBQADgYEAAwmbj3nvf1kwqu9otfrjCR27T4IGXTdf
# plKfFo3qHJIJRG71betYfDDo+WmNI3MLEm9Hqa45EfgqsZuwGsOO61mWAK3ODE2y
# 0DGmCFwqevzieh1XTKhlGOl5QGIllm7HxzdqgyEIjkHq3dlXPx13SYcqFgZepjhq
# IhKjURmDfrYwggSjMIIDi6ADAgECAhAOz/Q4yP6/NW4E2GqYGxpQMA0GCSqGSIb3
# DQEBBQUAMF4xCzAJBgNVBAYTAlVTMR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3Jh
# dGlvbjEwMC4GA1UEAxMnU3ltYW50ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBD
# QSAtIEcyMB4XDTEyMTAxODAwMDAwMFoXDTIwMTIyOTIzNTk1OVowYjELMAkGA1UE
# BhMCVVMxHTAbBgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMTQwMgYDVQQDEytT
# eW1hbnRlYyBUaW1lIFN0YW1waW5nIFNlcnZpY2VzIFNpZ25lciAtIEc0MIIBIjAN
# BgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAomMLOUS4uyOnREm7Dv+h8GEKU5Ow
# mNutLA9KxW7/hjxTVQ8VzgQ/K/2plpbZvmF5C1vJTIZ25eBDSyKV7sIrQ8Gf2Gi0
# jkBP7oU4uRHFI/JkWPAVMm9OV6GuiKQC1yoezUvh3WPVF4kyW7BemVqonShQDhfu
# ltthO0VRHc8SVguSR/yrrvZmPUescHLnkudfzRC5xINklBm9JYDh6NIipdC6Anqh
# d5NbZcPuF3S8QYYq3AhMjJKMkS2ed0QfaNaodHfbDlsyi1aLM73ZY8hJnTrFxeoz
# C9Lxoxv0i77Zs1eLO94Ep3oisiSuLsdwxb5OgyYI+wu9qU+ZCOEQKHKqzQIDAQAB
# o4IBVzCCAVMwDAYDVR0TAQH/BAIwADAWBgNVHSUBAf8EDDAKBggrBgEFBQcDCDAO
# BgNVHQ8BAf8EBAMCB4AwcwYIKwYBBQUHAQEEZzBlMCoGCCsGAQUFBzABhh5odHRw
# Oi8vdHMtb2NzcC53cy5zeW1hbnRlYy5jb20wNwYIKwYBBQUHMAKGK2h0dHA6Ly90
# cy1haWEud3Muc3ltYW50ZWMuY29tL3Rzcy1jYS1nMi5jZXIwPAYDVR0fBDUwMzAx
# oC+gLYYraHR0cDovL3RzLWNybC53cy5zeW1hbnRlYy5jb20vdHNzLWNhLWcyLmNy
# bDAoBgNVHREEITAfpB0wGzEZMBcGA1UEAxMQVGltZVN0YW1wLTIwNDgtMjAdBgNV
# HQ4EFgQURsZpow5KFB7VTNpSYxc/Xja8DeYwHwYDVR0jBBgwFoAUX5r1blzMzHSa
# 1N197z/b7EyALt0wDQYJKoZIhvcNAQEFBQADggEBAHg7tJEqAEzwj2IwN3ijhCcH
# bxiy3iXcoNSUA6qGTiWfmkADHN3O43nLIWgG2rYytG2/9CwmYzPkSWRtDebDZw73
# BaQ1bHyJFsbpst+y6d0gxnEPzZV03LZc3r03H0N45ni1zSgEIKOq8UvEiCmRDoDR
# EfzdXHZuT14ORUZBbg2w6jiasTraCXEQ/Bx5tIB7rGn0/Zy2DBYr8X9bCT2bW+IW
# yhOBbQAuOA2oKY8s4bL0WqkBrxWcLC9JG9siu8P+eJRRw4axgohd8D20UaF5Mysu
# e7ncIAkTcetqGVvP6KUwVyyJST+5z3/Jvz4iaGNTmr1pdKzFHTx/kuDDvBzYBHUw
# ggTQMIIDuKADAgECAgEHMA0GCSqGSIb3DQEBCwUAMIGDMQswCQYDVQQGEwJVUzEQ
# MA4GA1UECBMHQXJpem9uYTETMBEGA1UEBxMKU2NvdHRzZGFsZTEaMBgGA1UEChMR
# R29EYWRkeS5jb20sIEluYy4xMTAvBgNVBAMTKEdvIERhZGR5IFJvb3QgQ2VydGlm
# aWNhdGUgQXV0aG9yaXR5IC0gRzIwHhcNMTEwNTAzMDcwMDAwWhcNMzEwNTAzMDcw
# MDAwWjCBtDELMAkGA1UEBhMCVVMxEDAOBgNVBAgTB0FyaXpvbmExEzARBgNVBAcT
# ClNjb3R0c2RhbGUxGjAYBgNVBAoTEUdvRGFkZHkuY29tLCBJbmMuMS0wKwYDVQQL
# EyRodHRwOi8vY2VydHMuZ29kYWRkeS5jb20vcmVwb3NpdG9yeS8xMzAxBgNVBAMT
# KkdvIERhZGR5IFNlY3VyZSBDZXJ0aWZpY2F0ZSBBdXRob3JpdHkgLSBHMjCCASIw
# DQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBALngyxDUr3a91JNi6zBkuIEIbMME
# 2WIXji//PmXPj85i5jxSHNoWRUtVq3hrY4NikM4PaWyZyBoUi0zMRTPqiNyeo68r
# /oBhnXlXxM8u9D8wPF1H/JoWvMM3lkFRjhFLVPgovtCMvvAwOB7zsCb4Zkdjbd5x
# JkePOEdT0UYdtOPcAOpFrL28cdmqbwDb280wOnlPX0xH+B3vW8LEnWA7sbJDkdik
# M07qs9YnT60liqXG9NXQpq50BWRXiLVEVdQtKjo++Li96TIKApRkxBY6UPFKrud5
# M68MIAd/6N8EOcJpAmxjUvp3wRvIdIfIuZMYUFQ1S2lOvDvTSS4f3MHSUvsCAwEA
# AaOCARowggEWMA8GA1UdEwEB/wQFMAMBAf8wDgYDVR0PAQH/BAQDAgEGMB0GA1Ud
# DgQWBBRAwr0njsw0gzCiM9f7bLPwtCyAzjAfBgNVHSMEGDAWgBQ6moUHEGcotu/2
# vQVBbiDBlNoP3jA0BggrBgEFBQcBAQQoMCYwJAYIKwYBBQUHMAGGGGh0dHA6Ly9v
# Y3NwLmdvZGFkZHkuY29tLzA1BgNVHR8ELjAsMCqgKKAmhiRodHRwOi8vY3JsLmdv
# ZGFkZHkuY29tL2dkcm9vdC1nMi5jcmwwRgYDVR0gBD8wPTA7BgRVHSAAMDMwMQYI
# KwYBBQUHAgEWJWh0dHBzOi8vY2VydHMuZ29kYWRkeS5jb20vcmVwb3NpdG9yeS8w
# DQYJKoZIhvcNAQELBQADggEBAAh+bJMQyDi4lqmQS/+hX08E72w+nIgGyVCPpnP3
# VzEbvrzkL9v4utNb4LTn5nliDgyi12pjczG19ahIpDsILaJdkNe0fCVPEVYwxLZE
# nXssneVe5u8MYaq/5Cob7oSeuIN9wUPORKcTcA2RH/TIE62DYNnYcqhzJB61rCIO
# yheJYlhEG6uJJQEAD83EG2LbUbTTD1Eqm/S8c/x2zjakzdnYLOqum/UqspDRTXUY
# ij+KQZAjfVtL/qQDWJtGssNgYIP4fVBBzsKhkMO77wIv0hVU7kQV2Qqup4oz7bEt
# djYm3ATrn/dhHxXch2/uRpYoraEmfQoJpy4Eo428+LwEMAEwggUzMIIEG6ADAgEC
# AgkAlwn07EG7bckwDQYJKoZIhvcNAQELBQAwgbQxCzAJBgNVBAYTAlVTMRAwDgYD
# VQQIEwdBcml6b25hMRMwEQYDVQQHEwpTY290dHNkYWxlMRowGAYDVQQKExFHb0Rh
# ZGR5LmNvbSwgSW5jLjEtMCsGA1UECxMkaHR0cDovL2NlcnRzLmdvZGFkZHkuY29t
# L3JlcG9zaXRvcnkvMTMwMQYDVQQDEypHbyBEYWRkeSBTZWN1cmUgQ2VydGlmaWNh
# dGUgQXV0aG9yaXR5IC0gRzIwHhcNMTgwNTAyMTgwNjM3WhcNMjAwNjEyMTk0MTIz
# WjB0MQswCQYDVQQGEwJVUzEWMBQGA1UECBMNTWFzc2FjaHVzZXR0czEPMA0GA1UE
# BxMGQm9zdG9uMR0wGwYDVQQKExRQcm94eSBOZXR3b3JrcywgSW5jLjEdMBsGA1UE
# AxMUUHJveHkgTmV0d29ya3MsIEluYy4wggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAw
# ggEKAoIBAQDBajmeyJxUF2BLSCG5EI2XSjrkE3ac4LHOR4AJtSSnMFLiWQIph2Zf
# x5hIYB776GJFRXVieAPW+t3lmOf5RYIhbNc8xxIJ3/VSXZKBl7iTPDkc1EERwI8d
# Uw3mCw6QYkS2taY3pBhYSqTLloWeaWpETjj3euW5BZcSK3HKeHay6PpN92HhzdLL
# PTyAdH3TSr99QCLdyNzlTiSLaRMfEypZ87r7yK4QIU6sYKTSo7Tm1pUvkXeQIcQs
# tVnkAa3eeGFBjFQLQNpDhlZogwFipOJqWjiuDALO17+JoTgDggg63XRrVsgWdDow
# R2KGzwP6xXufP0McNJyGPOCxF1m9xP2BAgMBAAGjggGFMIIBgTAMBgNVHRMBAf8E
# AjAAMBMGA1UdJQQMMAoGCCsGAQUFBwMDMA4GA1UdDwEB/wQEAwIHgDA1BgNVHR8E
# LjAsMCqgKKAmhiRodHRwOi8vY3JsLmdvZGFkZHkuY29tL2dkaWcyczUtNC5jcmww
# XQYDVR0gBFYwVDBIBgtghkgBhv1tAQcXAjA5MDcGCCsGAQUFBwIBFitodHRwOi8v
# Y2VydGlmaWNhdGVzLmdvZGFkZHkuY29tL3JlcG9zaXRvcnkvMAgGBmeBDAEEATB2
# BggrBgEFBQcBAQRqMGgwJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmdvZGFkZHku
# Y29tLzBABggrBgEFBQcwAoY0aHR0cDovL2NlcnRpZmljYXRlcy5nb2RhZGR5LmNv
# bS9yZXBvc2l0b3J5L2dkaWcyLmNydDAfBgNVHSMEGDAWgBRAwr0njsw0gzCiM9f7
# bLPwtCyAzjAdBgNVHQ4EFgQUhXo2BaqNUEfl8vkqaW+RGSI/1JAwDQYJKoZIhvcN
# AQELBQADggEBAINdQ5UlRfjELvPD20WsfmQUTs6IxmAHBX87XaXv+OmhDTG2MJ8a
# cuCF3rsPFX2LNH+jQ8op4qlWdEjwyqdmgUh4b93gDI0RO1eo4b0/cuArcCW9tg7F
# lpwoi0zogmr/PcTzqql/flT64IQUFE9DpoENBnaXJyUJK2op5RixuxFeKRFcn6Vu
# hRlR5yAOydXtOf9vCcDrnLinlGtXGFhAKlOlXHf+vUsNH2tj4BJdOWzrtyf8dPDi
# EQNjil2EkBsIA6z3xvwN/rtqlirMaxZloYZjx61ZTqrnjZ7ZaHpDhw0ImhWoh9du
# HOh2qGF+YugB3jfE/VitbQWC4+GU5ncUqu8xggRzMIIEbwIBATCBwjCBtDELMAkG
# A1UEBhMCVVMxEDAOBgNVBAgTB0FyaXpvbmExEzARBgNVBAcTClNjb3R0c2RhbGUx
# GjAYBgNVBAoTEUdvRGFkZHkuY29tLCBJbmMuMS0wKwYDVQQLEyRodHRwOi8vY2Vy
# dHMuZ29kYWRkeS5jb20vcmVwb3NpdG9yeS8xMzAxBgNVBAMTKkdvIERhZGR5IFNl
# Y3VyZSBDZXJ0aWZpY2F0ZSBBdXRob3JpdHkgLSBHMgIJAJcJ9OxBu23JMAkGBSsO
# AwIaBQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEM
# BgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMCMGCSqG
# SIb3DQEJBDEWBBRDMxSuDBPN2NhoAZkDAfbeuVyxWjANBgkqhkiG9w0BAQEFAASC
# AQBKKfxxpF1AR28ixUIggukJfMCONU8FpOxscNYZNK0gxOxajQIFndvJ1MpdKHZ9
# Zm5ZFdXvIEAKpAWhMaBVhtl74YkcBzqN3aKDGxX26pcMuttv8mqw7oySnuEW9OLb
# YBhSnEkGnRdcZQbk2ZN/oAMJpHWZc7XJlaRYLJlYXYXRl8IATk2/mt086hkt1ha8
# cZAIrch5aW7b/frkgeQ3J2hhIPyyZaCGJE19jq1wZNkYa+ZOFHtysyDNrj1Q5wUa
# xBj7dlWfFiXBoTNsbr+dK2VxOcPK0oAesiSHRe/rOlMEKkI2zDzevem435k7blBa
# FwiGoUvG7AxsGSwHUH4wwWppoYICCzCCAgcGCSqGSIb3DQEJBjGCAfgwggH0AgEB
# MHIwXjELMAkGA1UEBhMCVVMxHTAbBgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9u
# MTAwLgYDVQQDEydTeW1hbnRlYyBUaW1lIFN0YW1waW5nIFNlcnZpY2VzIENBIC0g
# RzICEA7P9DjI/r81bgTYapgbGlAwCQYFKw4DAhoFAKBdMBgGCSqGSIb3DQEJAzEL
# BgkqhkiG9w0BBwEwHAYJKoZIhvcNAQkFMQ8XDTE4MTIxMjIwMDgxNlowIwYJKoZI
# hvcNAQkEMRYEFMtt13WGZSIc3By7ECDtkKmWmAm0MA0GCSqGSIb3DQEBAQUABIIB
# ADFIHGbNuDFhdYMTXBfYhpfT5R3Oh95D0TnYOn23mpMAQ9yKnrg2WP4dSu4Y0gdd
# QKCT9MEPlHAFspBS4RQRzZY1DAOeyP3pux2RFb39i/wkx/CER2uxisoQn2co4rEN
# sEJ2K+nfJgYRjCXhJXJemyQJo0oRv2Xe4FjMtdFqaoATPJrnGfUwR3HAlD/3LygR
# XtZGqpL1zSRIbRd/grnCA1/P3gYdLGf1LCnhqKIWPkL0+7glKvhLZT6hFSEgZN6i
# TWChlTa+fJ1SntKb/ll1nXf+5vCz9Y1oZGklulY9MwA//zpUwznoeSl6rMqi+vCy
# K/IW7ugD5dmN+v9nqulMFGk=
# SIG # End signature block
