############################################################################################
##
## Powershell Script for adding a new Personal User to the Proxy Gateway Server.
## It servers as an example on usage of "Import Account" functioanlity, which is a simple 
## and high-level way to manage Gateway's Users with associated security.
##
## Usage:
##
## PS> Import-Module Proxy-Gateway
## PS> Connect-Gateway -station 'example_gateway_station_name'
## PS> .\Add-NewPersonalUser.ps1 -Domain "example_domain_name" -Name "example_user_name" -WorkstationID "{GUID}""
##
############################################################################################

param([string]$Domain = $(throw "Please provide Domain")
    , [string]$Name = $(throw "Please provide User Name")
    , [string]$WorkstationID = $(throw "Please provide Workstation ID for the Personal user."))

# Imports
if (-not (Get-Module Proxy-Gateway))
{
    Import-Module Proxy-Gateway
}
if (-not (Get-Module Proxy-GatewaySec))
{
    Import-Module Proxy-GatewaySec
}
if (-not (Get-Module Proxy-Gateway-Utils))
{
    Import-Module Proxy-Gateway-Utils
}
if (-not (Get-Module Proxy-Constants))
{
    Import-Module Proxy-Constants
}

if (-not (Test-Connection))
{
    Write-Error -message "Error: There is no existing connection to the Gateway." 
    return
}

$gateway = Get-GatewayObject

#
# In order to add User to the Gateway, it need to be first found in one of the authorities
# which are associated with Proxy Identity Manager (PIM). You will need to choose which
# authority to use for looking up the User and you will need to use the exact name of this
# authority as it known to the PIM. In order to get the authority's exact name you can use
# `Get-GatewayAuthorities`, which will return a list of known names and choose one name from
# the list.
#

# List all authorities known to PIM and Gateway and choose one with the name matchig your domain name.
#
# NOTICE: `-match` is a regex string matching.
$auth = Get-GatewayAuthorities -gateway $gateway | Where-Object {$_ -match $Domain} | Select-Object -First 1
if (-not $auth)
{
    Write-Error -message "Error: No authority '$Domain' was found." 
    return
}

# Given authority's name and user's name, find this user in the authority and return object representing this user.
# `Find-UsersInAuthority` may return 0-N user objects. It is looking for the users using a "prefix" of the user name.
# If you give it an exact user name, as a prefix, it will most likely return just it. But there is a possibility to get more
# than one object, for example, if another user has the name which begins with the name of the user you are looking for.
#
$users, $more_data = Find-UsersInAuthority -gateway $gateway -authority $auth -prefix $Name -limit 0

if (-not $users -or $users.Count -eq 0)
{
    Write-Error -message "Error: No user '$Name' was found in '$Domain'." 
    return
}

if ($users.Count -gt 1)
{
    Write-Error -message "Error: More than one '$Name' was found in '$Domain'." 
    return
}

Add-GatewayAccount -gateway $gateway -account $users[0] -usercat $usercatPersonal -workstationID $WorkstationID

Write-Host "Success!"
