##############################################################################
##
## Power Shell Script for configuring Host(s) to report to Gateway(s)
##
## This sample script shows how to connect to a Gateway Server, walk the list
## of all managed Hosts, and use remote Host administration to configure them
## to remove obsolete Gateway configuration and to add a new Gateway configuration
## to the Host settings.
##
##
## Authentication Issues:
##
## A user who has rights to enumerate all (or even a subset) of the Hosts on a Gateway
## can use this script, but may not have access rights to reconfigure the Host settings.
## To work around this, you have a couple of choices:
## (1) Run the script as a member of "Domain Admins", or use the -credentials parameter
##     to specify the a domain administrator account and password (in the form "domain\account|password").
##     This works well if all machines belong to one domain.
## (2) Use the "RemoteControlGateway" account for authentication to the Host.  This
##     requires using another account with Gateway rights to connect to the Gateway, and
##     the RemoteControlGateway account for end-to-end authentication to the Host.
##     This works well if the machines belong to multiple domains, but there are trust
##     relationships among them, and the "RemoteControlGateway" account (with that exact name)
##     exists in one of the domains.  To specify different credential to the Host than the
##     Gateway, use the -credentials form "GWDOMAIN\account4gateway|password`nHOSTDOMAIN\account4host|password".
##     The "`n" syntax in PowerShell inserts a linefeed character, which delimits the two sets of credentials.
##     In the domain "MYDOMAIN", you can use the administrator account and RemoteControlGateway account
##     with the format: "MYDOMAIN\administrator|adminpassword`nMYDOMAIN\RemoteControlGateway|rcgpassword".
##
##
## Examples:
## 
## PS> .\ConfigureAllHostsOnGateway.ps1 -gateway 'localhost' -gatewayToRemove "old_gateway.domain.com" -gatewayToAdd "new_gateway.domain.com" -gatewayProtocolToAdd "WSS"
## 
##############################################################################

param([string]$gateway = $(throw "Please provide Gateway Identifier")
    , [string]$protocol ='TCP'
	, [string]$credentials = $null
	, [string]$gatewayToRemove
	, [string]$gatewayToAdd
	, [string]$gatewayProtocolToAdd)

# Imports
if (-not (Get-Module Proxy-Gateway))
{
    Import-Module Proxy-Gateway
}
if (-not (Get-Module Proxy-HostAdmin))
{
    Import-Module Proxy-HostAdmin
}

# Make sure we are not connected to Host or Gateway as we can only have one connection per Powershell session.
Disconnect-Host
Disconnect-Gateway

# Connect-Gateway returns $true or $false, which we discard here 
if (-not(Connect-Gateway -station $gateway -protocol $protocol -credentials $credentials))
{
	Write-Error -message "Connection to Gateway $gateway, failed."
	return
}

$gatewayObj = Get-GatewayObject

# For each host in the "All Hosts" group, connect Host Administration object & process it
foreach ($hh in $gatewayObj.getHosts())
{
	# Filter out Unamanaged, System Hosts and Hosts which never successfuly reported to the Gateway
	if (($hh.isManaged -eq 0) -or ($hh.protocol -eq 'internal') -or ($hh.address -eq ''))
	{
		continue
	}

	Write-Host "Processing Host: $($hh.machine)"

	$hostHA = Connect-Host -gateway $gateway -protocol $protocol -hostkey $hh.key -credentials $credentials
	if(-not $hostHA)
	{
		# Note: error reported by Connect-Host cmdlet
		# $msg = "Cannot connect to Host with key = " + $hh.key
		# Write-Error -message $msg
		continue
	}

	# HANDLE EXCEPTIONS HERE - if some machines shouldn't be touched, list them here by machine name
	# Example:
	if (($hh.machine -eq 'FILESERVER'))
	{
		Disconnect-Host
		continue
	}

	# First, remove the configurations that may already be set that are obsolete/not desired
	.\Remove-GatewayFromHostConfig.ps1 -gateway $gatewayToRemove

	# Finally, add in the configurations that are desired
	.\Add-GatewayToHostConfig.ps1 -gateway $gatewayToAdd -protocol $gatewayProtocolToAdd

	# Disconnect Host Administration object
	Disconnect-Host
}

Disconnect-Gateway
