##############################################################################
##
## Power Shell Script for exporting Host Settings as formatted reports.
##
## Example function ConvertTo-CustomObject(), converting Host Settings Object to custom object which can be used in Export-Csv or ConvertTo-XML cmdlets. 
## It's designed to produce 'human-readable' setting's representation in the contrast with Get-HostSettingsToXML.ps1, which
## outputs full, but very criptic report.
##
##
## Examples:
## 
## PS> .\Get-HostSettings.ps1 -gateway 'localhost' -hostkeys @('07C9B233-1BA8-4A48-81C8-2454E537C479') # Using logged-in user's credentials
## PS> .\Get-HostSettings.ps1 -gateway 'localhost' -hostkeys @('07C9B233-1BA8-4A48-81C8-2454E537C479') -credentials "domain\name|password"
## 
## PS> Set-ExecutionPolicy -Scope Process -executionPolicy RemoteSigned -Force
## PS> Import-Module Proxy-Gateway
## PS> Connect-Gateway 'localhost'
## PS> .\Get-HostSettings.ps1 -gateway 'localhost' -hostkeys @($gws.getHosts() | where { $_.machine -ne 'virtual' } | % { $_.workstationKey }) `
## PS> | Export-Csv -Path "hosts.csv" -Force -NoTypeInformation
## PS> Disconnect-Gateway
## 
##############################################################################

param(
	[string]$gateway = $(throw "Please provide Gateway Identifier")
	, $hostkeys = $(throw "Please provide one or multiple Host Key(s)")
    , [string]$protocol ='TCP'
    , [string]$credentials = '')

# Imports
if (-not (Get-Module Proxy-Utils))
{
    Import-Module Proxy-Utils
}
if (-not (Get-Module Proxy-HostAdmin))
{
    Import-Module Proxy-HostAdmin
}
if (-not (Get-Module Proxy-Constants))
{
    Import-Module Proxy-Constants
}

<#

.FUNCTION NAME

ConvertTo-CustomObject

.SYNOPSIS

Converts Host Settings Object to custom object which can be used in Export-Csv or ConvertTo-XML cmdlets. 

.EXAMPLE

PS > Import-Module Proxy-HostAdmin
PS > $hostHA = Connect-Host -gateway 'localhost' -hostkey "F4C03337-16D3-456E-BE43-A3EC8248DB4D"
PS > ConvertTo-CustomObject($hostHA.settings) | Export-Csv "host.csv"

#>
function ConvertTo-CustomObject($settings, [Boolean] $createHashTable=$false) {
    # Create hash-table object to fill with Host's settings.
    $retTable = @{}
    
    # This property gets or sets the station name of the Proxy Host.
    $retTable['stationName'] = [string] $settings.stationName

    # Controls whether a beep (or custom sound) is played when a connection is made or terminated.
    $retTable['beepOnConnect'] = [Boolean] $settings.beepOnConnect

    # Enables or disables a periodic beep (or custom sound) while a connection is in progress.
    $retTable['beepEvery'] = [Boolean] $settings.beepEvery

    # Sets or gets a value which controls whether the Host tray icon is displayed when there is no connection.
    $retTable['idleAppearance'] = $windowAppearance[$hostHA.settings.idleAppearance]

    # Sets or gets a value which controls whether the Host tray icon is displayed when a connection is active.
    $retTable['connectedAppearance'] = $windowAppearance[$hostHA.settings.connectedAppearance]

    # Indicates which Host notifications will be fired.
    $retTable['hostNotifications'] = $hostNotificationsMap[$hostHA.settings.hostNotifications]

    # Enables or disables the use of Windows security (authentication).
    $retTable['useWindowsSecurity'] = [Boolean] $hostHA.settings.useWindowsSecurity

    # Controls whether remote administration of the Host is allowed
    $retTable['allowRemoteAdmin'] = [Boolean] $hostHA.settings.allowRemoteAdmin

    # Enables or disables the lock workstation on disconnect feature. If this property is set to True, the workstation is locked when a remote control session ends.
    $retTable['lockWorkstationOnDisconnect'] = [Boolean] $hostHA.settings.lockWorkstationOnDisconnect

    # Enables or disables the reboot on disconnect feature. If this property is set to True, the Host computer will reboot when a remote control connection is terminated
    $retTable['reboot'] = [Boolean] $hostHA.settings.reboot

    # Sets or gets a value that controls whether local keyboard and mouse input is suppressed.
    $retTable['keySuppression'] = $suppressKeysMap[$hostHA.settings.keySuppression]

    # Sets or gets the Host access setting. The value of this property will be one of the values in the HostAccess enum.
    $retTable['access'] = $hostAccessMap[$hostHA.settings.access]

    # Sets or gets the Host permission setting. The value of this setting determines what type of permission is required to access the Host.
    $retTable['permission'] = $hostPermissionMap[$hostHA.settings.permission]

    # Sets or gets the number of seconds that are allowed to elapse while waiting for permission to connect to the Host. 
    # If the Host does not grant permission within this amount of time, permission is assumed to be denied, and the connection is terminated.
    $retTable['grantTime'] = $hostHA.settings.grantTime

    # Sets or gets the number of seconds that are allowed to elapse while requesting permission to connect to the Host. 
    # If the Host does not respond within this amount of time, permission is assumed to be granted, and the connection continues.
    $retTable['requestTime'] = $hostHA.settings.requestTime

    # Controls whether the Host console is automatically locked at connect time, if Connection Permission is requested from the Host, 
    # and the user at the console does not respond to the dialog prompt.
    $retTable['lockWorkstationOnDefault'] = [Boolean] $hostHA.settings.lockWorkstationOnDefault

    # Sets or gets the manageVisualEffects setting, which controls the how visual effects are managed on the Host.
    $retTable['manageVisualEffects'] = $manageVisualEffectsModeMap[$hostHA.settings.manageVisualEffects]

    # Sets or gets the list of Windows visual effects to be disabled when a connection is established. 
    # The value of this property can be any of the values in the VisualEffectFlags enum, or an OR of multiple VisualEffectFlags values.
    $retTable['visualEffects'] = $hostHA.settings.visualEffects

    # Allow to use UDP protocol for connections to this Host
    $retTable['protocolIP'] = $false
    $protocolIP  = $hostHA.settings.getProtocols() | ? { $_.protocolName -eq "IP" }
    if($protocolIP) { $retTable['protocolIP'] = $true }

    # Allow to use TCP protocol for connections to this Host
    $retTable['protocolTCP'] = $false
    $protocolTCP  = $hostHA.settings.getProtocols() | ? { $_.protocolName -eq "TCP" }
    if($protocolTCP) { $retTable['protocolTCP'] = $true }

    # Enables or disables the encryption of Host connections. This property must be set before a new connection is established.
    $retTable['useEncryption'] = [Boolean] $hostHA.settings.useEncryption

    # Set preffered ciphers to use with connection encription
    $retTable['preferredCiphers'] = $hostHA.settings.preferredCiphers

    # Determines whether remote control connections to this Host are required to be from Gateways in the list of configured Gateways. 
    # If this property is set to False, this Host can accept both Gateway and peer-to-peer connections.
    $retTable['connectByGateway'] = [Boolean] $hostHA.settings.connectByGateway

    # Controls whether the Host prefers User Mode Screen Capture as the screen capture technology, or prefers Kernel-mode screen capture.
    $retTable['preferUserMode'] = [Boolean] $hostHA.settings.preferUserMode

    # Set user-mode profile
    $retTable['currentUserModeProfile'] = $hostHA.settings.currentUserModeProfile.description
    
	if($createHashTable)
	{
		# Return as hash-table
		return $retTable
	}
	
	# Create custom object
	$retObj = New-Object PSObject
	foreach($key in $retTable.keys)
	{
		Add-Member NoteProperty -InputObject $retObj -Name $key -Value $retTable[$key]
	}
	
    return $retObj
}

# Connect Host Administration object
foreach($hostkey in $hostkeys)
{
	$hostHA = Connect-Host -gateway $gateway -hostkey $hostkey -protocol $protocol -credentials $credentials
	if(-not $hostHA)
	{
		Write-Error -message "Can't connect to Host: $hostkey" 
		continue
	}

	ConvertTo-CustomObject $hostHA.settings

	# Disconnect Host Administration object
	Disconnect-Host
}

