﻿<#
.SYNOPSIS

Creates a task which will connect to Host and execute codeblock on background. 
Writes report to output streams and errors to error stream. Close connection when done.

.EXAMPLE

PS > Invoke-HostTask -gateway "gateway.mydomain.com" -hostkey "w={B1D4803B-D1EC-433C-9FAC-0AFD8510CA42}" -protocol "TCP" -credentials "domain\user|password" -timeout 180 `
        -Task { 
            Param($prxHA) # This is how you get PrxHA object inside of the task
            
            $prxHA.settings.addGatewayConfigEx("TCP|2333", "gateway.mydomain.com", -1, 0, 0) | Out-Null
        }
PS > Invoke-HostTask -gateway "gateway.mydomain.com" -hostkey "w={B1D4803B-D1EC-433C-9FAC-0AFD8510CA42}" `  # connects with TCP protocol using current user identity of powershell and timeout 180sec
        -Task { 
            Param($prxHA) # This is how you get PrxHA object inside of the task
            
            $prxHA.settings.connectByGateway = 0
            $prxHA.settings.addGatewayConfigEx("TCP|2333", "gateway.mydomain.com", -1, 0, 0) | Out-Null
        }
PS > 
PS > get-job | ? { $_.Name.EndsWith("|c") } | % { Receive-Job $_ } # receive results from finished tasks
PS > get-job | ? { $_.Name.EndsWith("|f") } | % { Receive-Job $_ } # receive results from failed tasks
PS > 
#>

Param([string]$gateway
     	, [string]$hostkey
        , [string]$protocol='TCP'
        , [string]$credentials=''
        , [int]$timeout=180
        , [System.Management.Automation.ScriptBlock]$task = $(throw "Please provide '-Task' scriptblock."))

# Imports
if (-not (Get-Module Proxy-Utils))
{
    Import-Module Proxy-Utils
}
		
# Create a new Proxy SDK object of Proxy.ProxyHA type
$prxHA = New-Object -COM ProxySDKv10.PrxHA
if(-not $prxHA)
{ 
    Write-Error -message "Error: Failed to create Proxy.ProxyHA COM object." `
        -Category ObjectNotFound -ErrorId $_ERROR_ID_COMError
    
    return $false 
}
# Suppress UI messages from control
$prxHA.displayMessages = $false
# Create a set of unique event sources which can be distinguished by the suffix
$eventID = [System.Guid]::NewGuid().Guid

# Create cleaning action for PrxHA object, registered events and jobs
$cleaningAction = {
	Param($prxHA, $eventID)

    # Remove all event handlers: We don't do this now because we let user to gather results of the jobs and removing handlers also removes their jobs
    #(($eventID + "|c"), ($eventID + "|f"), ($eventID + "|d")) | `
    #    % { Unregister-Event -SourceIdentifier $_ -Force }

    # Remove all related jobs: We don't do this now because we let user to gather results of the jobs
    # (($eventID + "|c"), ($eventID + "|f"), ($eventID + "|d")) | `
    #    % { Remove-Job -Name $_ -Force }
    
	# Release all references to PrxHA COM object
    [System.Runtime.Interopservices.Marshal]::FinalReleaseComObject($prxHA) | Out-Null
}

# Create a new composite object to use inside of event's action
$eventData = New-Object PsObject -Property @{
    prxHA = $prxHA;
    eventID = $eventID;
    userTask = $task
	cleaningAction = $cleaningAction
	hostkey = $hostkey
    }

# Register an action for 'onConnectSucceeded' which is running user's "Task" and disconnect the object afterwards
Register-ObjectEvent $prxHA onConnectSucceeded -SourceIdentifier ($eventID + "|c") -MessageData $eventData `
    -Action { 
		# Distinguish between errors in User's task and in next call to PrxHA.disconnect()
		$tryingToDisconnect = $false
        
		# Set a 'trap' for errors in the User's Task. Strategy here is to trap any error 
		# in the Task and break it's execution. Then we try to close underlying connection
		# and if it failed, - release all resources.
		trap { 
			if($tryingToDisconnect) {
				"Host: $($event.MessageData.hostkey): Error trying to disconnect PrxHA object: $_"; 
				# Clean PrxHA object, registered events and jobs
				& ($event.MessageData.cleaningAction) -prxHA ($event.MessageData.prxHA) -eventID ($event.MessageData.eventID)
			} else {
				"Host: $($event.MessageData.hostkey): Error running User's Task: $_"; 
			}
		}
		
        # Invoke User's Task
        & ($event.MessageData.userTask) -prxHA ($event.MessageData.prxHA)
        
        # Disconnect the PrxHA object
		$tryingToDisconnect = $true
        ($event.MessageData.prxHA).disconnect()
		
		"Host: $($event.MessageData.hostkey): Finished."; 
    }
    
# Register an action for 'onConnectFailed' cleaning all events and jobs, and reporting an error
Register-ObjectEvent $prxHA onConnectFailed -SourceIdentifier ($eventID + "|f") -MessageData $eventData `
    -Action { 
		$errorText = "0x{0:X0}" -f $event.SourceArgs
        "Host: $($event.MessageData.hostkey): Error, connection failed: $errorText"
		
		# Clean PrxHA object, registered events and jobs
		& ($event.MessageData.cleaningAction) -prxHA ($event.MessageData.prxHA) -eventID ($event.MessageData.eventID)
    }

# Register an action for 'onConnectFailed' cleaning all events and jobs, and reporting disconnect
Register-ObjectEvent $prxHA onDisconnect -SourceIdentifier ($eventID + "|d") -MessageData $eventData `
    -Action { 
		# Clean PrxHA object, registered events and jobs
		& ($event.MessageData.cleaningAction) -prxHA ($event.MessageData.prxHA) -eventID ($event.MessageData.eventID)
    }
    
Write-Verbose "Invoking Task for Host: $hostkey"

# Initiate connection to the Proxy Host via the Gateway 
$prxHA.connectToHostViaGateway($protocol, $gateway, $hostkey, $credentials, $null) | Out-Null
$prxHA = $null

return $true
